@@ -86,14 +86,17 @@ class SignedData(pdf.PdfFileWriter):
             {
                 po.NameObject("/Size"): po.NumberObject(len(self._objects) + 1),
                 po.NameObject("/Root"): self.x_root,
-                po.NameObject("/Info"): self.x_info,
                 po.NameObject("/Prev"): po.NumberObject(prev.startxref),
                 po.NameObject("/ID"): self._ID,
             }
         )
+
         if prev.isEncrypted:
             trailer[po.NameObject("/Encrypt")] = prev.trailer.raw_get("/Encrypt")
 
+        if self.x_info:
+            trailer[po.NameObject("/Info")] = self.x_info
+
         if not prev.xrefstream:
             stream.write(pdf.b_("xref\n"))
             positions[0] = 1
@@ -281,8 +284,8 @@ class SignedData(pdf.PdfFileWriter):
             objapnref = self._addObject(objapn)
 
             for name in names + (
-                "Rect",
-                # "Subtype",
+                    "Rect",
+                    # "Subtype",
             ):
                 key = po.NameObject("/" + name)
                 v = pdfa[key]
@@ -309,7 +312,7 @@ class SignedData(pdf.PdfFileWriter):
             page0.update({po.NameObject("/Annots"): annots})
             self._objects[page0ref.idnum - 1] = page0
 
-        if "/Perms" not in catalog:
+        if udct.get("sigandcertify", False) and "/Perms" not in catalog:
             obj10 = po.DictionaryObject()
             obj10ref = self._addObject(obj10)
             obj11 = po.DictionaryObject()
@@ -320,7 +323,9 @@ class SignedData(pdf.PdfFileWriter):
             obj10.update(
                 {
                     po.NameObject("/Type"): po.NameObject("/TransformParams"),
-                    po.NameObject("/P"): po.NumberObject(2),
+                    po.NameObject("/P"): po.NumberObject(
+                        udct.get("sigflags", 3)
+                    ),
                     po.NameObject("/V"): po.NameObject("/1.2"),
                 }
             )
@@ -372,9 +377,9 @@ class SignedData(pdf.PdfFileWriter):
         x_root = prev.trailer.raw_get("/Root")
         self._objects[x_root.idnum - 1] = catalog
         self.x_root = po.IndirectObject(x_root.idnum, 0, self)
-        self.x_info = prev.trailer.raw_get("/Info")
+        self.x_info = prev.trailer.get("/Info")
 
-    def sign(self, datau, udct, key, cert, othercerts, algomd, hsm, timestampurl):
+    def sign(self, datau, udct, key, cert, othercerts, algomd, hsm, timestampurl, timestampcredentials=None):
         startdata = len(datau)
 
         fi = io.BytesIO(datau)
@@ -407,7 +412,7 @@ class SignedData(pdf.PdfFileWriter):
         else:
             md = getattr(hashlib, algomd)().digest()
             contents = signer.sign(
-                None, key, cert, othercerts, algomd, True, md, hsm, False, timestampurl
+                None, key, cert, othercerts, algomd, True, md, hsm, False, timestampurl, timestampcredentials
             )
             zeros = contents.hex().encode("utf-8")
 
@@ -459,13 +464,13 @@ class SignedData(pdf.PdfFileWriter):
         md = getattr(hashlib, algomd)()
         md.update(datau)
         b1 = datas[: br[1] - startdata]
-        b2 = datas[br[2] - startdata :]
+        b2 = datas[br[2] - startdata:]
         md.update(b1)
         md.update(b2)
         md = md.digest()
 
         contents = signer.sign(
-            None, key, cert, othercerts, algomd, True, md, hsm, False, timestampurl
+            None, key, cert, othercerts, algomd, True, md, hsm, False, timestampurl, timestampcredentials
         )
         contents = contents.hex().encode("utf-8")
         if aligned:
@@ -479,7 +484,8 @@ class SignedData(pdf.PdfFileWriter):
 
 
 def sign(
-    datau, udct, key, cert, othercerts, algomd="sha1", hsm=None, timestampurl=None
+        datau, udct, key, cert, othercerts, algomd="sha1", hsm=None, timestampurl=None,
+        timestampcredentials=None
 ):
     """
     parameters:
@@ -494,6 +500,9 @@ def sign(
             sigpage: int                default:0 - page on which signature should appear
             sigbutton: bool             default:False
             sigfield: string            default:Signature1
+            sigandcertify: bool         default:False
+                                                False - sign only document
+                                                True  - sign and certify document
             signaturebox: box|None      default:None - where to put signature image/string on selected page
             signature: string                   if box is not None then it should be latin1 encodable string
             signature_img: string|pil_image     if box is not None and string is None then it should be
@@ -512,9 +521,10 @@ def sign(
         algomd:string                   default: sha1 - name of the hashing algorithm used to calculate
                                             the hash of the document being signed e.g.: sha1, sha256, sha384, sha512, ripemd160
         hsm: an instance of endesive.hsm.HSM class used to sign using a hardware token or None
-        timestamputl: timestamp server URL or None
+        timestampurl: timestamp server URL or None
+        timestampcredentials:Dict username and password for authentication against timestamp server. Default: None
 
     returns: bytes ready for writing after unsigned pdf document containing its electronic signature
     """
     cls = SignedData()
-    return cls.sign(datau, udct, key, cert, othercerts, algomd, hsm, timestampurl)
+    return cls.sign(datau, udct, key, cert, othercerts, algomd, hsm, timestampurl, timestampcredentials)
